<?php
/*
Plugin Name: EDD custom checkout fields
Description: Add custom fields to edd checkout form
Plugin URI: http://wp-master.ir
Author: Omid Shamlu
Author URI: http://www.20script.ir
Version: 1.0
License: GPL2
Text Domain: ecf
 */
defined('ABSPATH') or die('No script kiddies please!');

$defs = array(
    'ecf_url' => plugin_dir_url(__FILE__),
    'ecf_dir' => plugin_dir_path(__FILE__),
    );
foreach ($defs as $def_name => $def_val) {
    define($def_name, $def_val);
}


/**
 * wp-master.ir class to handle custom checkout fields for edd
 */
class wpm_edd_custom_fields
{
    private static $instance = null;
    public $fields = array();
    public $prefix = 'edd-';


    public static function get_instance()
    {
        if (!isset(self::$instance)) {
            self::$instance = new self;
        }

        return self::$instance;
    }

    public function __construct()
    {
        /**
         * is EDD active?
         */
        if (!function_exists('edd_is_test_mode')) {
            return;
        }

        load_plugin_textdomain('ecf', false, dirname(plugin_basename(__FILE__)) . '/languages');
        __('EDD custom checkout fields','ecf');
        __('Add custom fields to edd checkout form','ecf');
        /**
         * admin menu
         */
        add_action( 'admin_menu', array($this,'_admin_menu') );

    }



    public function _admin_menu()
    {
        add_submenu_page('edit.php?post_type=download', __('checkout fields','ecf'), __('checkout fields','ecf'), 'manage_options', 'edd-custom-fields', array($this,'admin_menu') );
    }

    public function admin_menu()
    {
        require_once ecf_dir.'class-fields-generator.php';
        ?>
        <div class="wrap">
            <?php 
            ecf_fields_generator::get_instance('_ecf_custom_fields');
            ?>
        </div>
        <?php
    }

    public function make_fields($fields, $prefix = 'edd-')
    {
        if (empty($fields)) {
            return;
        }

        if (!empty($prefix)) {
            $this->prefix = $prefix;
        }
        $this->fields = $fields;
        add_action('edd_purchase_form_user_info_fields', array($this, 'wpm_edd_display_checkout_fields'));
        add_filter('edd_purchase_form_required_fields', array($this, 'wpm_edd_required_checkout_fields'));
        add_action('edd_checkout_error_checks', array($this, 'wpm_edd_validate_checkout_fields'), 10, 2);
        add_filter('edd_payment_meta', array($this, 'wpm_edd_store_custom_fields'));
        add_action('edd_payment_view_details', array($this, 'wpm_edd_view_order_details'), 999, 1);
        add_action('edd_add_email_tags', array($this, 'wpm_edd_add_sample_email_tag'));
        add_action('edd_insert_payment', array($this, 'wpm_edd_insert_payment'), 10, 2);

    }

    public function wpm_edd_display_checkout_fields()
    {
        $fields = $this->fields;
        foreach ($fields as $field) {
            $placeholder = empty($field['placeholder']) ? $field['title'] : $field['placeholder'];
            extract($field);
            $id = $this->prefix . $id;
            $required_class = ($required == 1) ? 'required' : '';
            $required_attr = ($required == 1) ? 'required="required"' : '';
            $required_star = ($required == 1) ? '<span class="edd-required-indicator">*</span>' : '';
            $placeholder = empty($placeholder) ? $title : $placeholder;
            echo '
            <p>
                <label class="edd-label" for="edd-' . $id . '">' . $title . $required_star . ' </label>
                <span class="edd-description">'.(isset($desc)?$desc:'').'</span>
                <input class="edd-input ' . $required_class . '" ' . $required_attr . ' type="text" name="edd_' . $id . '" id="edd-' . $id . '" placeholder="' . $placeholder . '" value=""/>
            </p>
            ';

        }

    }

    /**
     * Make phone number required
     * Add more required fields here if you need to
     */
    public function wpm_edd_required_checkout_fields($required_fields)
    {
        $fields = $this->fields;
        foreach ($fields as $field) {
            extract($field);
            if ($required == 1) {
                $id = $this->prefix . $id;
                $error = empty($error) ? __('Please check out this field', 'ecf') . ':' . $title : $error;

                $required_fields['edd_' . $id] = array(
                    'error_id' => 'invalid_' . $id,
                    'error_message' => $error,
                    );
            }
        }

        return $required_fields;
    }

    /**
     * Set error if phone number field is empty
     * You can do additional error checking here if required
     */
    public function wpm_edd_validate_checkout_fields($valid_data, $data)
    {
        $fields = $this->fields;
        foreach ($fields as $field) {
            extract($field);
            if ($required == 1) {
                $id = $this->prefix . $id;
                $error = empty($error) ? __('Please check out this field', 'ecf') . ':' . $title : $error;
                if (empty($data['edd_' . $id])) {
                    edd_set_error('invalid_' . $id, $error);
                }
            }
        }

    }

    /**
     * Store the custom field data into EDD's payment meta
     */
    public function wpm_edd_store_custom_fields($payment_meta)
    {
        if( did_action( 'edd_purchase' ) ) {
            $fields = $this->fields;
            foreach ($fields as $field) {
                extract($field);
                $id = $this->prefix . $id;
                $payment_meta[$id] = isset($_POST['edd_' . $id]) ? sanitize_text_field($_POST['edd_' . $id]) : '';
            }
        }
        return $payment_meta;
    }

    /**
     * Add the phone number to the "View Order Details" page
     */
    public function wpm_edd_view_order_details($payment_id)
    {
        $payment_meta = (edd_get_payment_meta($payment_id));
        $_payment_meta = get_post_meta($payment_id, '_wpm_edd_custom_fields');

        $fields = $this->fields;
        foreach ($fields as $field) {
            extract($field);
            $id = $this->prefix . $id;
            $_val = isset($payment_meta[$id]) ? $payment_meta[$id] : '';
            if (is_array($_payment_meta) && isset($_payment_meta[$id])) {
                $_val = empty($_payment_meta[$id]) ? '-' : $_payment_meta[$id];

            }
            if (empty($_val)) {$_val = '-';}
            echo '
            <div class="column-container">
                <div class="column">
                    <strong>' . $title . ': </strong>
                    ' . $_val . '
                </div>
            </div>
            ';
        }

    }
    public function wpm_edd_add_sample_email_tag()
    {
        $fields = $this->fields;

        try {
            foreach ($fields as $field) {
                extract($field);
                $id = $this->prefix . $id;
                edd_add_email_tag($id, $title, function ($payment_id) {
                    $payment_data = edd_get_payment_meta($payment_id);
                    return $payment_data[$id];
                });
            }
        } catch (Exception $e) {
            //Nothing
            //echo 'Caught exception: ',  $e->getMessage(), "\n";
        }

    }
    /**
     * ensure data store while some plugins override added meta values
     */
    public function wpm_edd_insert_payment($payment_id, $payment_data)
    {
        $payment_meta = (edd_get_payment_meta($payment_id));
        $_payment_meta = array();
        $fields = $this->fields;
        foreach ($fields as $field) {
            extract($field);
            $id = $this->prefix . $id;
            if (isset($payment_meta[$id])) {
                $_payment_meta[$id] = $payment_meta[$id];
            }
        }
        if (!empty($_payment_meta)) {
            update_post_meta($payment_id, '_wpm_edd_custom_fields', $_payment_meta);
        }

    }

}

$ecf = wpm_edd_custom_fields::get_instance();
$_ecf_custom_fields = get_option('_ecf');
unset($_ecf_custom_fields['last_saved']);
if(is_array($_ecf_custom_fields) && !empty($_ecf_custom_fields))
{
    $_fields = array();
    foreach ($_ecf_custom_fields['type'] as $index => $value) {
        switch ($_ecf_custom_fields['type'][$index]) {
            case 'text':
            $_fields[] = 
            array(
                'id' => 'ecf_'.$index,
                'title' => $_ecf_custom_fields['name'][$index] ,
                'required' => ($_ecf_custom_fields['required'][$index]==1)?1:0, // 1 - 0
                'placeholder' => $_ecf_custom_fields['text_placeholder'][$index],
                'desc' => $_ecf_custom_fields['desc'][$index],
                'type' => 'text', //text
                'error' => '', //
            );
            break;
            
        }

        if(!empty($_fields))
        {
          $ecf->make_fields($_fields);  
      }
  }
}
